/* AC Current and Kilowatt/Hr calculator

  by Glen Popiel - KW5GP

  Uses Adafruit_ST7735 and Adafruit+GFX libraries


*/

#define debug
#define baud_rate 115200

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library

#define Current_Pin A0  // Define the Current Sense pin
#define zero_point 420  // Define the Zero Setpoint
#define max_point 509 // Define the High (15A) setpoint

#define TFT_CS     10
#define TFT_DC     9
#define TFT_RST    8

#define tft_delay 10  // set the TFT command delay to 10ms

float watts, amps, cost_day;  // Variables for wattage, amperage, and the estimated cost per day
int count;  // The A/D count of the Current Sense pin

// Change these for your location
const float voltage = 119.1;  // The AC Line voltage
const float cost_kwh = 0.1183; // Cost per Kilowatt/Hour

// Instantiate the TFT Display
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Start");
#endif

  // Use this initializer if you're using a 1.8" TFT
  tft.initR(INITR_BLACKTAB);   // initialize a ST7735S chip, black tab
  delay(tft_delay);
  tft.fillScreen(ST7735_BLUE); // Clear the display
  delay(tft_delay);
  tft.setRotation(1); // Set the screen rotation
  delay(tft_delay);
  tft.setTextWrap(false);
  delay(tft_delay);
  tft.setTextSize(2);
  delay(tft_delay);
  tft.setTextColor(ST7735_GREEN);
  delay(tft_delay);
  tft.setCursor(50, 30);
  delay(tft_delay);
  tft.print("KW5GP");
  delay(tft_delay);
  tft.setCursor(20, 60);
  delay(tft_delay);
  tft.print("AC Current");
  delay(tft_delay);
  tft.setCursor(40, 80);
  delay(tft_delay);
  tft.print("Monitor");
  delay(tft_delay);

  delay (5000);

  tft.fillScreen(ST7735_BLUE);
  delay(tft_delay);
  tft.setTextSize(1);
  delay(tft_delay);

  tft.setCursor(30, 10);
  delay(tft_delay);
  tft.print("AC Current Monitor");
}

void loop()
{
  count = analogRead(Current_Pin);  // Read the incoming current sense voltage

#ifdef debug
  Serial.print("A/D Count: ");
  Serial.print(count);
#endif

  count = count - zero_point; // Subtract the zero point value

#ifdef debug
  Serial.print("   Adjusted A/D Count: ");
  Serial.print(count);
#endif

  if (count < 0) // Our count should be zero, but if it's negative, make it zero
  {
    count = 0;
  }

  amps = map(count, 0, max_point, 0, 1500); // Map the current sense voltage to hundredths of an Amp
  amps = amps / 100; // Divide the current by 100 to place the decimal point

#ifdef debug
  Serial.print("   Amps: ");
  Serial.print(amps);
#endif

  tft.setCursor(5, 50);
  delay(tft_delay);
  tft.print("Amps: ");
  delay(tft_delay);
  tft.print(amps, 1);
  delay(tft_delay);
  tft.setCursor(5, 70);
  delay(tft_delay);
  tft.print("Watts: ");
  delay(tft_delay);
  watts = (amps * voltage); // Calculate and display the wattage

#ifdef debug
  Serial.print("   Watts: ");
  Serial.print(watts);
#endif

  tft.print(watts, 1);
  delay(tft_delay);
  tft.setCursor(5, 90);
  delay(tft_delay);
  tft.print("Cost/Day: ");
  delay(tft_delay);
  cost_day = ((watts / 1000) * cost_kwh) * 24; // Calculate and display the estimated Kilowatt/Hour cost per day
  tft.print(cost_day, 2);
  delay(tft_delay);

#ifdef debug
  Serial.print("   Cost/Day: ");
  Serial.println(cost_day);
#endif

  delay(5000);

  clear_data();
  /*

    // KW/hrs = cost per KW/h * power
    lcd.print("Cost/day: ");
    cost_day = ((watts / 1000) * cost_kwh) * 24; // Calculate and display the estimated Kilowatt/Hour cost per day
    lcd.print(cost_day, 2);
    delay(5000);
  */
}

void clear_data() //Clears the data area of the display
{
  tft.fillRect(0, 30, 159, 90, ST7735_BLUE); // Clear the CAT data area
  delay(tft_delay);
}
