/*
  ARRL RF Radiation Exposure Monitor
  written by Glen Popiel - KW5GP

*/
#include <Adafruit_NeoPixel.h>

// #define debug  // Enables Debug Mode - Sends Debug output to Serial Monitor
#define baud_rate 115200

#define NUM_LEDS 8
#define LED_PIN 5
#define RF_input_pin A0
#define gain_pin A1
#define led_Red 255,0,0
#define led_Green 0,255,0
#define led_Yellow 255,255,0
#define led_brightness 50
#define update_delay 200


// The noise and gain definition values were determined empirically - change to suit as desired

int signal_strength = 0;
int effective_strength = 0;
int max_strength = 1000;
int LEDsToLight = 0;
int rf_gain;

Adafruit_NeoPixel pixels(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);

void setup()
{

#ifdef debug
  Serial.begin(baud_rate);  // Start the Serial port
#endif

  pixels.begin(); // INITIALIZE NeoPixel strip object 

  pixels.clear(); // Set all pixel colors to 'off'
  pixels.show();

}
void loop()
{

  int rf_gain = analogRead(gain_pin);
  
#ifdef debug
  Serial.print("Sensitivity: ");
  Serial.print(rf_gain);
#endif
  // Read adc
  signal_strength = analogRead(RF_input_pin);  // Read Analog input

#ifdef debug
  Serial.print("   Signal Strength: "); // display ADC read status in debug mode
  Serial.print(signal_strength);  // Display ADC value in debug mode
#endif

  effective_strength = map(signal_strength, 0, max_strength, 0, rf_gain*4);

#ifdef debug
  Serial.print("   Effective Strength: "); // display ADC read status in debug mode
  Serial.print(effective_strength);  // Display ADC value in debug mode
#endif

  // Convert to LED amplitude
  LEDsToLight = map(effective_strength, 0, max_strength, 0, NUM_LEDS);

  if (LEDsToLight < 0)
  {
    LEDsToLight = 0;
  }

  if (LEDsToLight > NUM_LEDS)
  {
    LEDsToLight = NUM_LEDS;
  }

#ifdef debug
  Serial.print("   LEDs to Light: ");
  Serial.println(LEDsToLight);
#endif

// Clear existing LEDs
  pixels.clear();
  pixels.setBrightness(led_brightness);
  
    for (int led = 0; led < LEDsToLight; led = led + 1)
    {
      if (led <= 3)
      {
    pixels.setPixelColor(led, pixels.Color(led_Green));
      }
      if (led > 3 && led <= 6)
      {
        pixels.setPixelColor(led, pixels.Color(led_Yellow));
      }
      if (led > 5)
      {
        pixels.setPixelColor(led, pixels.Color(led_Red));
      }
    }
    pixels.show();
    delay(update_delay);
}
