/* KW5GP Station Monitor

  Displays Voltage, Current, Date and Time with threshold warnings

  Uses DS3231 Library from Rinkydinkelectronics.com

  Uses Adafruit_ST7735 and Adafruit+GFX libraries

*/

#define debug
#define baud_rate 115200

#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_ST7735.h> // Hardware-specific library
#include <SPI.h>
#include <DS3231.h>

// For the breakout, you can use any 2 or 3 pins
// These pins will also work for the 1.8" TFT shield
#define TFT_CS     10
#define TFT_DC     9
#define TFT_RST    8  // you can also connect this to the Arduino reset
// in which case, set this #define pin to -1

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS,  TFT_DC, TFT_RST);
#define voltage_pin A1
#define current_pin A0

// Init the DS3231
DS3231 rtc(SDA, SCL);

float current_cal_0 = 3.0;
int current_cal_max = 1009;


float voltage, current;
float volt_drop = 0.776;
int volt_cal = 2480;

float current_high = 25;
float voltage_low = 11.9;
float voltage_high = 14.0;
float temperature;
String rtcdate, rtctime, rtcday, rtcmonth, rtcyear ;

void setup()
{
#ifdef debug
  Serial.begin(baud_rate);
  Serial.println("Start");
#endif

  // Use this initializer if you're using a 1.8" TFT
  tft.initR(INITR_BLACKTAB);   // initialize a ST7735S chip, black tab

  // Initialize the rtc object
  rtc.begin();

  // The following lines can be uncommented to set the time
  // rtc.setDOW(FRIDAY);        // Set Day-of-Week to SUNDAY
  // rtc.setTime(11,12,0);     // Set the time - format is h:m:s (24hr format)
  // rtc.setDate(4,8,2023);   // Set the date - format is dd.mm.yyyy

  tft.fillScreen(ST7735_BLUE); // Clear the display
  tft.setRotation(1); // Set the screen rotation
  tft.setTextWrap(false);
  tft.setTextSize(2);
  tft.setTextColor(ST7735_GREEN);
  tft.setCursor(50, 30);
  tft.print("KW5GP");
  tft.setCursor(37, 60);
  tft.print("Station");
  tft.setCursor(37, 80);
  tft.print("Monitor");
  delay (5000);

  tft.fillScreen(ST7735_BLUE);
}

void loop()
{
  tft.setTextSize(3);
  tft.setTextColor(ST7735_GREEN);
  tft.fillScreen(ST7735_BLUE);
  tft.setCursor(40, 5);
  tft.print("KW5GP");

  do
  {
    tft.setTextSize(3);
    tft.setCursor(30, 40);

    readCurrent();
    readVoltage();
    tft.setTextColor(ST7735_GREEN, ST7735_BLUE);
    if (current >= current_high)
    {
      tft.setTextColor(ST7735_RED, ST7735_BLUE);
    }
    tft.print(current, 2);
    tft.print("A  ");
    tft.setCursor(30, 70);
    tft.setTextColor(ST7735_GREEN, ST7735_BLUE);
    if (voltage <= voltage_low)
    {
      tft.setTextColor(ST7735_YELLOW, ST7735_BLUE);
    }
    else
    {
      if (voltage >= voltage_high)
      {
        tft.setTextColor(ST7735_RED, ST7735_BLUE);
      }
    }

    tft.print(voltage, 2);
    tft.print("V  ");
    tft.setTextColor(ST7735_GREEN, ST7735_BLUE);
    tft.setTextSize(2);
    tft.setCursor(55, 100);
    temperature = rtc.getTemp();
    temperature = (temperature * 1.8) + 32;

#ifdef debug
    Serial.print("Temperature: ");
    Serial.print(temperature);
#endif

    tft.print(temperature, 1);
    tft.print(" F ");
    tft.setTextSize(1);
    tft.setCursor(0, 120);
    rtcdate = rtc.getDateStr();
    rtcmonth = rtcdate.substring(3, 5);
    rtcday = rtcdate.substring(0, 2);
    rtcyear = rtcdate.substring(6, 10);
    rtctime = rtc.getTimeStr();
    rtctime = rtctime.substring(0, 5);
    tft.setCursor(0, 120);
    tft.print(" ");
    tft.print(rtc.getDOWStr());
    tft.print(" ");
    tft.print(rtcmonth);
    tft.print("/");
    tft.print(rtcday);
    tft.print("/");
    tft.print(rtcyear);
    tft.print(" ");
    tft.print(rtctime);

#ifdef debug
    Serial.print("    Day: ");
    Serial.print(rtc.getDOWStr());
    Serial.print("    Date: ");
    Serial.print(rtcmonth + "/" + rtcday + "/" + rtcyear);
    Serial.print("    Time: ");
    Serial.println(rtctime);
#endif

    delay(10000);

  } while (true);
}

void readVoltage()
{
  float count = analogRead(voltage_pin);

#ifdef debug
  Serial.print("    Voltage A/D count: ");
  Serial.print(count);
#endif

  voltage = map(count, 0, 1023, 0, volt_cal);


  //  voltage = random(0,1500); // For display test

  voltage = (voltage / 100) + volt_drop; // Map the Voltage A/D value to voltage

#ifdef debug
  Serial.print("    Calculated Voltage: ");
  Serial.println(voltage);
#endif

}

void readCurrent()
{
  float count = analogRead(current_pin);  // read the Current value

#ifdef debug
  Serial.print("Current A/D count: ");
  Serial.print(count);
#endif

  count = count + current_cal_0; // set the zero center point at approx 2.5V
  current = map(count, 0, current_cal_max, -3000, 3000); // convert it to Amps XX.X
  //current = count;
  //  current = random(0,3000); // For display test

  current = current / 100;

#ifdef debug
  Serial.print("    Calculated Current: ");
  Serial.print(current);
#endif
}
