/*
  Touch Tone Encoder
  By Glen Popiel - KW5GP
  KW5GP@ARRL.NET

  Uses Adafruit Keypad library - must copy keypad.config.h from
  Arduino>Libraries>Adafruit_Keypad>Examples>keypad_test to
  Arduino Libraries>Adafruit_Keypad folder

  HT9200 section is derived from HT9200B DTMF Generator sketch by BroHogan 7/17/09
  SETUP FOR PARALLEL MODE - Requires 5 pins - 4 data + CS (to stop tone)

  Select the Adafruit Product ID for your keypad.
     PID3844 4x4 Matrix Keypad - This also works for the standard 4 x 4 Membrane Keypad
     PID3845 3x4 Matrix Keypad
     PID1824 3x4 Phone-style Matrix Keypad
     PID1332 Membrane 1x4 Keypad
     PID419  Membrane 3x4 Matrix Keypad
*/

// 
#define debug

#include "Adafruit_Keypad.h"

// define your specific keypad here via PID
#define KEYPAD_PID3844  // This keypad also works with the 4x4 Membrane keypad

// define the keypad pins
#define R1    5
#define R2    4
#define R3    3
#define R4    2
#define C1    9
#define C2    8
#define C3    7
#define C4    6

// leave this import after the above configuration
#include "keypad_config.h"

// Define Tone Special Digits
#define ZERO      10
#define STAR      11
#define POUND     12
#define A_KEY     13
#define B_KEY     14
#define C_KEY     15
#define D_KEY      0

char key_char = ' ';
int digit;

// Define the HT9200 Pins
#define D0_PIN  10              // Data 0
#define D1_PIN  11              // Data 1
#define D2_PIN  12              // Data 2
#define D3_PIN  13              // Data 3
#define CS_PIN  14              // uses Analog pin A0 as a digital I/O pin for Chip Select LOW = On


//initialize an instance of class NewKeypad
Adafruit_Keypad kpd = Adafruit_Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS);

void setup()
{
#ifdef debug
  Serial.begin(115200);
#endif

  kpd.begin();

  // Set the pin modes for the HT9200B
  pinMode(D0_PIN, OUTPUT);
  pinMode(D1_PIN, OUTPUT);
  pinMode(D2_PIN, OUTPUT);
  pinMode(D3_PIN, OUTPUT);
  pinMode(CS_PIN, OUTPUT);
  turn_off_tone();

#ifdef debug
  delay(3000);
  Serial.println(" ");
  Serial.println("Start");
#endif
}

void loop()
{
  // put your main code here, to run repeatedly:
  kpd.tick();

  while (kpd.available()) // Check if keypad button is pressed
  {
    keypadEvent e = kpd.read(); // get the keypad button
    key_char = (char)e.bit.KEY;

#ifdef debug
    Serial.print(key_char);
#endif

    if (e.bit.EVENT == KEY_JUST_PRESSED)
    {

#ifdef debuf
      Serial.println(" pressed");
#endif

      send_key_tone();
    }
    else if (e.bit.EVENT == KEY_JUST_RELEASED)
    {

#ifdef debug
      Serial.print(" released");
#endif

      turn_off_tone();
    }
  }

  delay(10);
}


void send_key_tone()
{
  convert_key();

  // Turn on the tone for the selected key

#ifdef debug
  Serial.print("  Sending tone: ");
  Serial.print(key_char);
  Serial.print(" Decimal: ");
  Serial.println(key_char, DEC);
#endif

  digitalWrite(D0_PIN, bitRead(digit, 0));

#ifdef debug
  Serial.print(" D0: ");
  Serial.println(bitRead(digit, 0));
#endif

  digitalWrite(D1_PIN, bitRead(digit, 1));

  #ifdef debug
  Serial.print(" D1: ");
  Serial.println(bitRead(digit, 1));
  #endif
  
  digitalWrite(D2_PIN, bitRead(digit, 2));

  #ifdef debug
  Serial.print(" D2: ");
  Serial.println(bitRead(digit, 2));
  #endif
  
  digitalWrite(D3_PIN, bitRead(digit, 3));

  #ifdef debug
  Serial.print(" D3: ");
  Serial.println(bitRead(digit, 3));
  #endif
  
  delay(100);
  digitalWrite(CS_PIN, LOW);           // Enable the tone encoder
}

void turn_off_tone()
{
  // Turn off the tone
  #ifdef debug
  Serial.println("  Turning off tone");
  #endif

  digitalWrite(CS_PIN, HIGH);           // Disable chip (req if parallel)
  digitalWrite(D0_PIN, LOW);
  digitalWrite(D1_PIN, LOW);
  digitalWrite(D2_PIN, LOW);
  digitalWrite(D3_PIN, LOW);
  delay(100);
}

void convert_key()
{

  digit = int(key_char);

  if (digit >= 49 && digit <= 57)
  {
    digit = digit - 48;
  }
  else
  {
    switch (key_char)
    {
      case '0':
        digit = ZERO;
        break;

      case 'A':
        digit = A_KEY;
        break;

      case 'B':
        digit = B_KEY;
        break;

      case 'C':
        digit = C_KEY;
        break;

      case 'D':
        digit = D_KEY;
        break;

      case '*':
        digit = STAR;
        break;

      case '#':
        digit = POUND;
        break;
    }
  }

  #ifdef debug
  Serial.print("Digit = ");
  Serial.println(digit, HEX);
  #endif
  
}
